@@ -9,7 +9,6 @@ import jwt
 import pkg_resources
 import requests
 from email_validator import EmailNotValidError, validate_email
-from jwt.exceptions import ExpiredSignatureError
 
 from descope.common import (
     COOKIE_DATA_NAME,
@@ -20,13 +19,17 @@ from descope.common import (
     SESSION_TOKEN_NAME,
     DeliveryMethod,
     EndpointsV1,
+    EndpointsV2,
 )
 from descope.exceptions import (
+    API_RATE_LIMIT_RETRY_AFTER_HEADER,
+    ERROR_TYPE_API_RATE_LIMIT,
     ERROR_TYPE_INVALID_ARGUMENT,
     ERROR_TYPE_INVALID_PUBLIC_KEY,
     ERROR_TYPE_INVALID_TOKEN,
     ERROR_TYPE_SERVER_ERROR,
     AuthException,
+    RateLimitException,
 )
 
 
@@ -73,6 +76,20 @@ class Auth:
                 kid, pub_key, alg = self._validate_and_load_public_key(public_key)
                 self.public_keys = {kid: (pub_key, alg)}
 
+    def _raise_rate_limit_exception(self, response):
+        resp = response.json()
+        raise RateLimitException(
+            resp.get("errorCode", "429"),
+            ERROR_TYPE_API_RATE_LIMIT,
+            resp.get("errorDescription", ""),
+            resp.get("errorMessage", ""),
+            rate_limit_parameters={
+                API_RATE_LIMIT_RETRY_AFTER_HEADER: int(
+                    response.headers.get(API_RATE_LIMIT_RETRY_AFTER_HEADER, 0)
+                )
+            },
+        )
+
     def do_get(
         self,
         uri: str,
@@ -88,6 +105,8 @@ class Auth:
             verify=self.secure,
         )
         if not response.ok:
+            if response.status_code == 429:
+                self._raise_rate_limit_exception(response)  # Raise RateLimitException
             raise AuthException(
                 response.status_code, ERROR_TYPE_SERVER_ERROR, response.text
             )
@@ -105,6 +124,9 @@ class Auth:
             params=params,
         )
         if not response.ok:
+            if response.status_code == 429:
+                self._raise_rate_limit_exception(response)  # Raise RateLimitException
+
             raise AuthException(
                 response.status_code, ERROR_TYPE_SERVER_ERROR, response.text
             )
@@ -128,30 +150,30 @@ class Auth:
 
     @staticmethod
     def verify_delivery_method(
-        method: DeliveryMethod, identifier: str, user: dict
+        method: DeliveryMethod, login_id: str, user: dict
     ) -> bool:
-        if not identifier:
+        if not login_id:
             return False
 
-        if not user or not isinstance(user, dict):
+        if not isinstance(user, dict):
             return False
 
         if method == DeliveryMethod.EMAIL:
             if not user.get("email", None):
-                user["email"] = identifier
+                user["email"] = login_id
             try:
                 validate_email(email=user["email"], check_deliverability=False)
                 return True
             except EmailNotValidError:
                 return False
-        elif method == DeliveryMethod.PHONE:
+        elif method == DeliveryMethod.SMS:
             if not user.get("phone", None):
-                user["phone"] = identifier
+                user["phone"] = login_id
             if not re.match(PHONE_REGEX, user["phone"]):
                 return False
         elif method == DeliveryMethod.WHATSAPP:
             if not user.get("phone", None):
-                user["phone"] = identifier
+                user["phone"] = login_id
             if not re.match(PHONE_REGEX, user["phone"]):
                 return False
         else:
@@ -164,7 +186,7 @@ class Auth:
         suffix = ""
         if method is DeliveryMethod.EMAIL:
             suffix = "email"
-        elif method is DeliveryMethod.PHONE:
+        elif method is DeliveryMethod.SMS:
             suffix = "sms"
         elif method is DeliveryMethod.WHATSAPP:
             suffix = "whatsapp"
@@ -176,11 +198,11 @@ class Auth:
         return f"{base}/{suffix}"
 
     @staticmethod
-    def get_identifier_by_method(method: DeliveryMethod, user: dict) -> Tuple[str, str]:
+    def get_login_id_by_method(method: DeliveryMethod, user: dict) -> Tuple[str, str]:
         if method is DeliveryMethod.EMAIL:
             email = user.get("email", "")
             return "email", email
-        elif method is DeliveryMethod.PHONE:
+        elif method is DeliveryMethod.SMS:
             phone = user.get("phone", "")
             return "phone", phone
         elif method is DeliveryMethod.WHATSAPP:
@@ -191,6 +213,19 @@ class Auth:
                 400, ERROR_TYPE_INVALID_ARGUMENT, f"Unknown delivery method: {method}"
             )
 
+    @staticmethod
+    def get_method_string(method: DeliveryMethod) -> str:
+        if method is DeliveryMethod.EMAIL:
+            return "email"
+        elif method is DeliveryMethod.SMS:
+            return "phone"
+        elif method is DeliveryMethod.WHATSAPP:
+            return "whatsapp"
+        else:
+            raise AuthException(
+                400, ERROR_TYPE_INVALID_ARGUMENT, f"Unknown delivery method: {method}"
+            )
+
     @staticmethod
     def validate_email(email: str):
         if email == "":
@@ -221,20 +256,13 @@ class Auth:
                 400, ERROR_TYPE_INVALID_ARGUMENT, "Invalid phone number"
             )
 
-        if method != DeliveryMethod.PHONE and method != DeliveryMethod.WHATSAPP:
+        if method != DeliveryMethod.SMS and method != DeliveryMethod.WHATSAPP:
             raise AuthException(
                 400, ERROR_TYPE_INVALID_ARGUMENT, "Invalid delivery method"
             )
 
-    def refresh_token(self, refresh_token: str) -> dict:
-        uri = Auth._compose_refresh_token_url()
-        response = self.do_post(uri, {}, None, refresh_token)
-
-        resp = response.json()
-        return self.generate_jwt_response(resp, refresh_token)
-
     def exchange_access_key(self, access_key: str) -> dict:
-        uri = Auth._compose_exchange_access_key_url()
+        uri = EndpointsV1.exchange_auth_access_key_path
         server_response = self.do_post(uri, {}, None, access_key)
         json = server_response.json()
         return self._generate_auth_info(json, None, False)
@@ -294,15 +322,16 @@ class Auth:
             )
 
     def _fetch_public_keys(self) -> None:
-
         # This function called under mutex protection so no need to acquire it once again
         response = requests.get(
-            f"{self.base_url}{EndpointsV1.publicKeyPath}/{self.project_id}",
+            f"{self.base_url}{EndpointsV2.public_key_path}/{self.project_id}",
             headers=self._get_default_headers(),
             verify=self.secure,
         )
 
         if not response.ok:
+            if response.status_code == 429:
+                self._raise_rate_limit_exception(response)  # Raise RateLimitException
             raise AuthException(
                 response.status_code,
                 ERROR_TYPE_SERVER_ERROR,
@@ -311,7 +340,8 @@ class Auth:
 
         jwks_data = response.text
         try:
-            jwkeys = json.loads(jwks_data)
+            jwkeys_wrapper = json.loads(jwks_data)
+            jwkeys = jwkeys_wrapper["keys"]
         except Exception as e:
             raise AuthException(
                 500, ERROR_TYPE_INVALID_PUBLIC_KEY, f"Unable to load jwks. Error: {e}"
@@ -480,53 +510,68 @@ class Auth:
         claims["jwt"] = token
         return claims
 
-    def _validate_and_load_tokens(self, session_token: str, refresh_token: str) -> dict:
-        if not session_token and not refresh_token:
+    def validate_session(self, session_token: str) -> dict:
+        if not session_token:
             raise AuthException(
-                500,
+                400,
                 ERROR_TYPE_INVALID_TOKEN,
-                "Both refresh token and session token are empty",
+                "Session token is required for validation",
             )
 
-        if session_token:
-            try:
-                return self._validate_token(session_token)
-            except ExpiredSignatureError:
-                # Session token expired, check that refresh token is valid
-                if refresh_token:
-                    try:
-                        self._validate_token(refresh_token)
-                    except Exception as e:
-                        raise AuthException(
-                            401, ERROR_TYPE_INVALID_TOKEN, f"Invalid refresh token: {e}"
-                        )
-                else:
-                    raise AuthException(
-                        401,
-                        ERROR_TYPE_INVALID_TOKEN,
-                        "Session token expired and no refresh token provided",
-                    )
-                # Refresh token is valid now refresh the session token
-                return self.refresh_token(refresh_token)  # return jwt_response dict
-            except Exception as e:
-                raise AuthException(
-                    500, ERROR_TYPE_INVALID_TOKEN, f"Invalid token: {e}"
-                )
+        try:
+            res = self._validate_token(session_token)
+            return self.adjust_properties(res, True)
+        except RateLimitException as e:
+            raise e
+        except Exception as e:
+            raise AuthException(
+                401, ERROR_TYPE_INVALID_TOKEN, f"Invalid session token: {e}"
+            )
+
+    def refresh_session(self, refresh_token: str) -> dict:
+        if not refresh_token:
+            raise AuthException(
+                400,
+                ERROR_TYPE_INVALID_TOKEN,
+                "Refresh token is required to refresh a session",
+            )
 
-        # If we got here, we did not have a session token so only do the refresh
         try:
             self._validate_token(refresh_token)
+        except RateLimitException as e:
+            raise e
         except Exception as e:
+            # Refresh is invalid
             raise AuthException(
                 401, ERROR_TYPE_INVALID_TOKEN, f"Invalid refresh token: {e}"
             )
-        # Refresh token is valid now refresh the session token
-        return self.refresh_token(refresh_token)  # return jwt_response dict
+
+        uri = EndpointsV1.refresh_token_path
+        response = self.do_post(uri, {}, None, refresh_token)
+
+        resp = response.json()
+        return self.generate_jwt_response(resp, refresh_token)
+
+    def validate_and_refresh_session(
+        self, session_token: str = None, refresh_token: str = None
+    ) -> dict:
+        if not session_token and not refresh_token:
+            raise AuthException(
+                400,
+                ERROR_TYPE_INVALID_TOKEN,
+                "Session and refresh tokens are both empty",
+            )
+
+        try:
+            return self.validate_session(session_token)
+        except Exception:
+            # Session is invalid - try to refresh it
+            return self.refresh_session(refresh_token)
 
     @staticmethod
-    def _compose_refresh_token_url() -> str:
-        return EndpointsV1.refreshTokenPath
-
-    @staticmethod
-    def _compose_exchange_access_key_url() -> str:
-        return EndpointsV1.exchangeAuthAccessKeyPath
+    def extract_masked_address(response: dict, method: DeliveryMethod) -> str:
+        if method == DeliveryMethod.SMS or method == DeliveryMethod.WHATSAPP:
+            return response["maskedPhone"]
+        elif method == DeliveryMethod.EMAIL:
+            return response["maskedEmail"]
+        return ""
