@@ -3,22 +3,22 @@ from descope.common import (
     REFRESH_SESSION_COOKIE_NAME,
     EndpointsV1,
     LoginOptions,
-    validateRefreshTokenProvided,
+    validate_refresh_token_provided,
 )
 from descope.exceptions import ERROR_TYPE_INVALID_ARGUMENT, AuthException
 
 
-class WebauthN:
+class WebAuthn:
     _auth: Auth
 
     def __init__(self, auth):
         self._auth = auth
 
-    def sign_up_start(self, identifier: str, origin: str, user: dict = None) -> dict:
+    def sign_up_start(self, login_id: str, origin: str, user: dict = None) -> dict:
         """
         Docs
         """
-        if not identifier:
+        if not login_id:
             raise AuthException(
                 400, ERROR_TYPE_INVALID_ARGUMENT, "Identifier cannot be empty"
             )
@@ -28,17 +28,20 @@ class WebauthN:
                 400, ERROR_TYPE_INVALID_ARGUMENT, "Origin cannot be empty"
             )
 
-        uri = EndpointsV1.signUpAuthWebauthnStart
-        body = WebauthN._compose_sign_up_start_body(identifier, user, origin)
+        if not user:
+            user = {}
+
+        uri = EndpointsV1.sign_up_auth_webauthn_start_path
+        body = WebAuthn._compose_sign_up_start_body(login_id, user, origin)
         response = self._auth.do_post(uri, body)
 
         return response.json()
 
-    def sign_up_finish(self, transactionID: str, response: str) -> dict:
+    def sign_up_finish(self, transaction_id: str, response: str) -> dict:
         """
         Docs
         """
-        if not transactionID:
+        if not transaction_id:
             raise AuthException(
                 400, ERROR_TYPE_INVALID_ARGUMENT, "Transaction id cannot be empty"
             )
@@ -48,8 +51,8 @@ class WebauthN:
                 400, ERROR_TYPE_INVALID_ARGUMENT, "Response cannot be empty"
             )
 
-        uri = EndpointsV1.signUpAuthWebauthnFinish
-        body = WebauthN._compose_sign_up_in_finish_body(transactionID, response)
+        uri = EndpointsV1.sign_up_auth_webauthn_finish_path
+        body = WebAuthn._compose_sign_up_in_finish_body(transaction_id, response)
         response = self._auth.do_post(uri, body, None, "")
 
         resp = response.json()
@@ -60,15 +63,15 @@ class WebauthN:
 
     def sign_in_start(
         self,
-        identifier: str,
+        login_id: str,
         origin: str,
-        loginOptions: LoginOptions = None,
-        refreshToken: str = None,
+        login_options: LoginOptions = None,
+        refresh_token: str = None,
     ) -> dict:
         """
         Docs
         """
-        if not identifier:
+        if not login_id:
             raise AuthException(
                 400, ERROR_TYPE_INVALID_ARGUMENT, "Identifier cannot be empty"
             )
@@ -78,11 +81,11 @@ class WebauthN:
                 400, ERROR_TYPE_INVALID_ARGUMENT, "Origin cannot be empty"
             )
 
-        validateRefreshTokenProvided(loginOptions, refreshToken)
+        validate_refresh_token_provided(login_options, refresh_token)
 
-        uri = EndpointsV1.signInAuthWebauthnStart
-        body = WebauthN._compose_sign_in_start_body(identifier, origin, loginOptions)
-        response = self._auth.do_post(uri, body, pswd=refreshToken)
+        uri = EndpointsV1.sign_in_auth_webauthn_start_path
+        body = WebAuthn._compose_sign_in_start_body(login_id, origin, login_options)
+        response = self._auth.do_post(uri, body, pswd=refresh_token)
 
         return response.json()
 
@@ -100,8 +103,8 @@ class WebauthN:
                 400, ERROR_TYPE_INVALID_ARGUMENT, "Response cannot be empty"
             )
 
-        uri = EndpointsV1.signInAuthWebauthnFinish
-        body = WebauthN._compose_sign_up_in_finish_body(transaction_id, response)
+        uri = EndpointsV1.sign_in_auth_webauthn_finish_path
+        body = WebAuthn._compose_sign_up_in_finish_body(transaction_id, response)
         response = self._auth.do_post(uri, body, None)
 
         resp = response.json()
@@ -112,13 +115,13 @@ class WebauthN:
 
     def sign_up_or_in_start(
         self,
-        identifier: str,
+        login_id: str,
         origin: str,
     ) -> dict:
         """
         Docs
         """
-        if not identifier:
+        if not login_id:
             raise AuthException(
                 400, ERROR_TYPE_INVALID_ARGUMENT, "Identifier cannot be empty"
             )
@@ -128,17 +131,17 @@ class WebauthN:
                 400, ERROR_TYPE_INVALID_ARGUMENT, "Origin cannot be empty"
             )
 
-        uri = EndpointsV1.signUpOrInAuthWebauthnStart
-        body = WebauthN._compose_sign_up_or_in_start_body(identifier, origin)
+        uri = EndpointsV1.sign_up_or_in_auth_webauthn_start_path
+        body = WebAuthn._compose_sign_up_or_in_start_body(login_id, origin)
         response = self._auth.do_post(uri, body)
 
         return response.json()
 
-    def update_start(self, identifier: str, refresh_token: str, origin: str):
+    def update_start(self, login_id: str, refresh_token: str, origin: str):
         """
         Docs
         """
-        if not identifier:
+        if not login_id:
             raise AuthException(
                 400, ERROR_TYPE_INVALID_ARGUMENT, "Identifier cannot be empty"
             )
@@ -148,8 +151,8 @@ class WebauthN:
                 400, ERROR_TYPE_INVALID_ARGUMENT, "Refresh token cannot be empty"
             )
 
-        uri = EndpointsV1.updateAuthWebauthnStart
-        body = WebauthN._compose_update_start_body(identifier, origin)
+        uri = EndpointsV1.update_auth_webauthn_start_path
+        body = WebAuthn._compose_update_start_body(login_id, origin)
         response = self._auth.do_post(uri, body, None, refresh_token)
 
         return response.json()
@@ -168,13 +171,13 @@ class WebauthN:
                 400, ERROR_TYPE_INVALID_ARGUMENT, "Response cannot be empty"
             )
 
-        uri = EndpointsV1.updateAuthWebauthnFinish
-        body = WebauthN._compose_update_finish_body(transaction_id, response)
+        uri = EndpointsV1.update_auth_webauthn_finish_path
+        body = WebAuthn._compose_update_finish_body(transaction_id, response)
         self._auth.do_post(uri, body)
 
     @staticmethod
-    def _compose_sign_up_start_body(identifier: str, user: dict, origin: str) -> dict:
-        body = {"user": {"externalId": identifier}}
+    def _compose_sign_up_start_body(login_id: str, user: dict, origin: str) -> dict:
+        body = {"user": {"loginId": login_id}}
         if user is not None:
             for key, val in user.items():
                 body["user"][key] = val
@@ -183,18 +186,18 @@ class WebauthN:
 
     @staticmethod
     def _compose_sign_in_start_body(
-        identifier: str, origin: str, loginOptions: LoginOptions = None
+        login_id: str, origin: str, login_options: LoginOptions = None
     ) -> dict:
         return {
-            "externalId": identifier,
+            "loginId": login_id,
             "origin": origin,
-            "loginOptions": loginOptions.__dict__ if loginOptions else {},
+            "loginOptions": login_options.__dict__ if login_options else {},
         }
 
     @staticmethod
-    def _compose_sign_up_or_in_start_body(identifier: str, origin: str) -> dict:
+    def _compose_sign_up_or_in_start_body(login_id: str, origin: str) -> dict:
         return {
-            "externalId": identifier,
+            "loginId": login_id,
             "origin": origin,
         }
 
@@ -203,8 +206,8 @@ class WebauthN:
         return {"transactionId": transaction_id, "response": response}
 
     @staticmethod
-    def _compose_update_start_body(identifier: str, origin: str) -> dict:
-        body = {"externalId": identifier}
+    def _compose_update_start_body(login_id: str, origin: str) -> dict:
+        body = {"loginId": login_id}
         if origin:
             body["origin"] = origin
         return body
