@@ -7,10 +7,11 @@ from descope.authmethod.enchantedlink import EnchantedLink  # noqa: F401
 from descope.authmethod.magiclink import MagicLink  # noqa: F401
 from descope.authmethod.oauth import OAuth  # noqa: F401
 from descope.authmethod.otp import OTP  # noqa: F401
+from descope.authmethod.password import Password  # noqa: F401
 from descope.authmethod.saml import SAML  # noqa: F401
 from descope.authmethod.totp import TOTP  # noqa: F401
-from descope.authmethod.webauthn import WebauthN  # noqa: F401
-from descope.common import SESSION_TOKEN_NAME, EndpointsV1
+from descope.authmethod.webauthn import WebAuthn  # noqa: F401
+from descope.common import EndpointsV1
 from descope.exceptions import ERROR_TYPE_INVALID_ARGUMENT, AuthException
 from descope.mgmt import MGMT  # noqa: F401
 
@@ -34,7 +35,8 @@ class DescopeClient:
         self._saml = SAML(auth)
         self._otp = OTP(auth)
         self._totp = TOTP(auth)
-        self._webauthn = WebauthN(auth)
+        self._webauthn = WebAuthn(auth)
+        self._password = Password(auth)
 
     @property
     def mgmt(self):
@@ -72,6 +74,10 @@ class DescopeClient:
     def webauthn(self):
         return self._webauthn
 
+    @property
+    def password(self):
+        return self._password
+
     def validate_permissions(self, jwt_response: dict, permissions: List[str]) -> bool:
         """
         Validate that a jwt_response has been granted the specified permissions.
@@ -162,16 +168,49 @@ class DescopeClient:
                 return False
         return True
 
-    def validate_session_request(self, session_token: str, refresh_token: str) -> dict:
+    def validate_session(self, session_token: str) -> dict:
         """
-        Validate the session for a given request. If the user is authenticated but the
-            session has expired, the session token will automatically be refreshed.
-        Either the session_token or the refresh_token must be provided.
-        Call this function every time you make a private API call that requires an authorized
-            user.
+        Validate a session token. Call this function for every incoming request to your
+        private endpoints. Alternatively, use validate_and_refresh_session in order to
+        automatically refresh expired sessions.
 
         Args:
-        session_token (str): The session token, which contains the signature that will be validated
+        session_token (str): The session token to be validated
+
+        Return value (dict):
+        Return dict includes the session token and all JWT claims
+
+        Raise:
+        AuthException: Exception is raised if session is not authorized or any other error occurs
+        """
+        return self._auth.validate_session(session_token)
+
+    def refresh_session(self, refresh_token: str) -> dict:
+        """
+        Refresh a session. Call this function when a session expires and needs to be refreshed.
+
+        Args:
+        refresh_token (str): The refresh token that will be used to refresh the session
+
+        Return value (dict):
+        Return dict includes the session token, refresh token, and all JWT claims
+
+        Raise:
+        AuthException: Exception is raised if refresh token is not authorized or any other error occurs
+        """
+        return self._auth.refresh_session(refresh_token)
+
+    def validate_and_refresh_session(
+        self, session_token: str, refresh_token: str
+    ) -> dict:
+        """
+        Validate the session token and refresh it if it has expired, the session token will automatically be refreshed.
+        Either the session_token or the refresh_token must be provided.
+        Call this function for every incoming request to your
+        private endpoints. Alternatively, use validate_session to only validate the session.
+
+        Args:
+        session_token (str): The session token to be validated
         refresh_token (str): The refresh token that will be used to refresh the session token, if needed
 
         Return value (dict):
@@ -180,18 +219,7 @@ class DescopeClient:
         Raise:
         AuthException: Exception is raised if session is not authorized or another error occurs
         """
-        res = self._auth._validate_and_load_tokens(
-            session_token, refresh_token
-        )  # return jwt_response dict
-
-        # Check if we had to refresh the session token and got a new one
-        if res.get(SESSION_TOKEN_NAME, None) and session_token != res.get(
-            SESSION_TOKEN_NAME
-        ).get("jwt"):
-            return res
-        else:
-            # In such case we return only the data related to the session token
-            return self._auth.adjust_properties({SESSION_TOKEN_NAME: res}, True)
+        return self._auth.validate_and_refresh_session(session_token, refresh_token)
 
     def logout(self, refresh_token: str) -> requests.Response:
         """
@@ -213,7 +241,7 @@ class DescopeClient:
                 f"signed refresh token {refresh_token} is empty",
             )
 
-        uri = EndpointsV1.logoutPath
+        uri = EndpointsV1.logout_path
         return self._auth.do_post(uri, {}, None, refresh_token)
 
     def logout_all(self, refresh_token: str) -> requests.Response:
@@ -236,19 +264,19 @@ class DescopeClient:
                 f"signed refresh token {refresh_token} is empty",
             )
 
-        uri = EndpointsV1.logoutAllPath
+        uri = EndpointsV1.logout_all_path
         return self._auth.do_post(uri, {}, None, refresh_token)
 
     def me(self, refresh_token: str) -> dict:
         """
         Retrieve user details for the refresh token. The returned data includes email, name, phone,
-            list of externalIds and boolean flags for verifiedEmail, verifiedPhone.
+            list of loginIds and boolean flags for verifiedEmail, verifiedPhone.
 
         Args:
         refresh_token (str): The refresh token
 
         Return value (dict): returns the user details from the server
-            (email:str, name:str, phone:str, externalIds[str], verifiedEmail:bool, verifiedPhone:bool)
+            (email:str, name:str, phone:str, loginIds[str], verifiedEmail:bool, verifiedPhone:bool)
 
         Raise:
         AuthException: Exception is raised if session is not authorized or another error occurs
@@ -260,24 +288,10 @@ class DescopeClient:
                 f"signed refresh token {refresh_token} is empty",
             )
 
-        uri = EndpointsV1.mePath
+        uri = EndpointsV1.me_path
         response = self._auth.do_get(uri, None, None, refresh_token)
         return response.json()
 
-    def refresh_token(self, refresh_token: str) -> dict:
-        """
-        Return a new session token for the given refresh token
-
-        Args:
-        refresh_token (str): The refresh token
-
-        Return value (dict): returns the session token from the server
-
-        Raise:
-        AuthException: Exception is raised if session is not authorized or another error occurs
-        """
-        return self._auth.refresh_token(refresh_token)
-
     def exchange_access_key(self, access_key: str) -> dict:
         """
         Return a new session token for the given access key
